@@ -73,6 +73,7 @@
 #define _SLANG_BC_SET_INTRIN_LVALUE	0x33
 #define _SLANG_BC_SET_STRUCT_LVALUE	0x34
 #define _SLANG_BC_FIELD			0x35
+#define _SLANG_BC_SET_ARRAY_LVALUE	0x36
 
 #define _SLANG_BC_LINE_NUM		0x40
 
@@ -273,6 +274,7 @@ extern int _SLroll_stack (int);
 /* If argument *p is positive, the top |*p| objects on the stack are rolled
  * up.  If negative, the stack is rolled down.
  */
+extern int _SLdup_n (int n);
 
 extern SLang_Array_Type *_SLang_apropos (char *, char *, unsigned int);
 extern void _SLang_implements_intrinsic (char *);
@@ -314,7 +316,8 @@ extern int _SLinit_slcomplex (void);
 
 extern int _SLang_init_slstrops (void);
 extern int _SLstrops_do_sprintf_n (int);
-
+extern int _SLang_sscanf (void);
+extern double _SLang_atof (char *);
 extern int _SLang_init_bstring (void);
 extern int _SLang_init_sltime (void);
 extern void _SLpack (void);
@@ -338,8 +341,12 @@ extern int _SLdo_pop (void);
 extern unsigned int _SLsys_getkey (void);
 extern int _SLsys_input_pending (int);
 #ifdef IBMPC_SYSTEM
-extern unsigned int _SLpc_convert_scancode (unsigned int);
+extern unsigned int _SLpc_convert_scancode (unsigned int, unsigned int, int);
+#define _SLTT_KEY_SHIFT		1
+#define _SLTT_KEY_CTRL		2
+#define _SLTT_KEY_ALT		4
 #endif
+
 #ifdef REAL_UNIX_SYSTEM
 extern int SLtt_tigetflag (char *, char **);
 #endif
