@@ -7,8 +7,8 @@
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
  */
-#define SLANG_VERSION 10309	       /* 1.3.8 */
-#define SLANG_VERSION_STRING "1.3.9"
+#define SLANG_VERSION 10310	       /* 1.3.10 */
+#define SLANG_VERSION_STRING "B1.3.10"
 
 /*{{{ System Dependent Macros and Typedefs */
 
@@ -127,11 +127,7 @@ typedef int (*FVOID_STAR)(void);
 # else
 #  define SLFREE(x) free((char *)(x))
 #  define SLMALLOC malloc
-#  if defined(__cplusplus) && !defined(__BEOS__)
-#   define SLREALLOC(p,n) realloc((malloc_t) (p), (n))
-#  else
-#   define SLREALLOC realloc
-#  endif
+#  define SLREALLOC realloc
 #  define SLCALLOC calloc
 # endif
 #endif
@@ -1216,6 +1212,11 @@ extern int SLtt_Try_Termcap;
 extern int SLtt_Msdos_Cheap_Video;
 #endif
 
+typedef unsigned short SLsmg_Char_Type;
+#define SLSMG_EXTRACT_CHAR(x) ((x) & 0xFF)
+#define SLSMG_EXTRACT_COLOR(x) (((x)>>8)&0xFF)
+#define SLSMG_BUILD_CHAR(ch,color) (((SLsmg_Char_Type)(unsigned char)(ch))|((color)<<8))
+
 extern int SLtt_flush_output (void);
 extern void SLtt_set_scroll_region(int, int);
 extern void SLtt_reset_scroll_region(void);
@@ -1232,7 +1233,7 @@ extern void SLtt_normal_video(void);
 extern void SLtt_cls(void);
 extern void SLtt_beep(void);
 extern void SLtt_reverse_index(int);
-extern void SLtt_smart_puts(unsigned short *, unsigned short *, int, int);
+extern void SLtt_smart_puts(SLsmg_Char_Type *, SLsmg_Char_Type *, int, int);
 extern void SLtt_write_string (char *);
 extern void SLtt_putchar(char);
 extern int SLtt_init_video (void);
@@ -1326,7 +1327,7 @@ extern void SLsmg_touch_screen (void);
 extern int SLsmg_init_smg (void);
 extern int SLsmg_reinit_smg (void);
 extern void SLsmg_reset_smg (void);
-extern unsigned short SLsmg_char_at(void);
+extern SLsmg_Char_Type SLsmg_char_at(void);
 extern void SLsmg_set_screen_start (int *, int *);
 extern void SLsmg_draw_hline (unsigned int);
 extern void SLsmg_draw_vline (int);
@@ -1335,9 +1336,9 @@ extern void SLsmg_draw_box (int, int, unsigned int, unsigned int);
 extern int SLsmg_get_column(void);
 extern int SLsmg_get_row(void);
 extern void SLsmg_forward (int);
-extern void SLsmg_write_color_chars (unsigned short *, unsigned int);
-extern unsigned int SLsmg_read_raw (unsigned short *, unsigned int);
-extern unsigned int SLsmg_write_raw (unsigned short *, unsigned int);
+extern void SLsmg_write_color_chars (SLsmg_Char_Type *, unsigned int);
+extern unsigned int SLsmg_read_raw (SLsmg_Char_Type *, unsigned int);
+extern unsigned int SLsmg_write_raw (SLsmg_Char_Type *, unsigned int);
 extern void SLsmg_set_color_in_region (int, int, int, unsigned int, unsigned int);
 extern int SLsmg_Display_Eight_Bit;
 extern int SLsmg_Tab_Width;
@@ -1453,7 +1454,7 @@ typedef struct
    void (*tt_delete_nlines)(int);
    void (*tt_cls) (void);
    void (*tt_del_eol) (void);
-   void (*tt_smart_puts) (unsigned short *, unsigned short *, int, int);
+   void (*tt_smart_puts) (SLsmg_Char_Type *, SLsmg_Char_Type *, int, int);
    int (*tt_flush_output) (void);
    int (*tt_reset_video) (void);
    int (*tt_init_video) (void);
@@ -1733,7 +1734,7 @@ extern void SLadd_at_handler (long *, char *);
     {(n),NULL, SLANG_DCONSTANT, (val)}
 
 #ifndef offsetof
-# define offsetof(T,F) ((unsigned int)((char *)&((T *)0)->F))
+# define offsetof(T,F) ((unsigned int)((char *)&((T *)0L)->F - (char *)0L))
 #endif
 #define MAKE_ISTRUCT_FIELD(s,f,n,t,r) {(n), offsetof(s,f), (t), (r)}
 
