@@ -427,11 +427,21 @@ int _SLang_push_datatype (unsigned char data_type)
 static int datatype_deref (unsigned char type, VOID_STAR ptr)
 {
    SLang_Class_Type *cl;
+   int status;
+
+   /* The parser generated code for this as if a function call were to be
+    * made.  However, we are calling the deref object routine
+    * instead of the function call.  So, I must simulate the function call.
+    */
+   if (-1 == _SL_increment_frame_pointer ())
+     return -1;
 
    type = (unsigned char) *(int *) ptr;
    cl = _SLclass_get_class (type);
+   status = (*cl->cl_datatype_deref) (type);
 
-   return (*cl->cl_datatype_deref) (type);
+   (void) _SL_decrement_frame_pointer ();
+   return status;
 }
 
 static int datatype_push (unsigned char type_unused, VOID_STAR ptr)
