@@ -1360,7 +1360,7 @@ static int bce_color_eqs (unsigned int a, unsigned int b)
      return Ansi_Color_Map[a].fgbg == Ansi_Color_Map[b].fgbg;
    
    /* If either are color 0, then we do not know what that means since the
-    * terminal oes not support BCE */
+    * terminal does not support BCE */
    if ((a == 0) || (b == 0))
      return 0;
      
@@ -1405,11 +1405,11 @@ static void write_string_with_care (char *str)
    tt_write (str, len);
 }
 
-static void send_attr_str (unsigned short *s)
+static void send_attr_str (SLsmg_Char_Type *s)
 {
    unsigned char out[256], ch, *p;
    register SLtt_Char_Type attr;
-   register unsigned short sh;
+   register SLsmg_Char_Type sh;
    int color, last_color = -1;
 
    p = out;
@@ -1499,20 +1499,20 @@ static void forward_cursor (unsigned int n, int row)
 }
 
 
-void SLtt_smart_puts(unsigned short *neww, unsigned short *oldd, int len, int row)
+void SLtt_smart_puts(SLsmg_Char_Type *neww, SLsmg_Char_Type *oldd, int len, int row)
 {
-   register unsigned short *p, *q, *qmax, *pmax, *buf;
-   unsigned short buffer[256];
+   register SLsmg_Char_Type *p, *q, *qmax, *pmax, *buf;
+   SLsmg_Char_Type buffer[256];
    unsigned int n_spaces;
-   unsigned short *space_match, *last_buffered_match;
+   SLsmg_Char_Type *space_match, *last_buffered_match;
 #ifdef HP_GLITCH_CODE
    int handle_hp_glitch = 0;
 #endif
-   unsigned short space_char;
+   SLsmg_Char_Type space_char;
 #define SLTT_USE_INSERT_HACK 1
 #if SLTT_USE_INSERT_HACK
-   unsigned short insert_hack_prev = 0;
-   unsigned short insert_hack_char = 0;
+   SLsmg_Char_Type insert_hack_prev = 0;
+   SLsmg_Char_Type insert_hack_char = 0;
 
    if ((row + 1 == SLtt_Screen_Rows)
        && (len == SLtt_Screen_Cols)
@@ -1528,12 +1528,6 @@ void SLtt_smart_puts(unsigned short *neww, unsigned short *oldd, int len, int ro
      }
 #endif
      
-#if SLTT_HAS_NON_BCE_SUPPORT
-   space_char = 0x20; /*  | (Bce_Color_Offset << 8); */
-#else
-   space_char = 0x20;
-#endif
-
    q = oldd; p = neww;
    qmax = oldd + len;
    pmax = p + len;
@@ -1575,7 +1569,7 @@ void SLtt_smart_puts(unsigned short *neww, unsigned short *oldd, int len, int ro
 #ifdef HP_GLITCH_CODE
    if (Has_HP_Glitch)
      {
-	unsigned short *qq = q;
+	SLsmg_Char_Type *qq = q;
 
 	SLtt_goto_rc (row, (int) (p - neww));
 
@@ -1595,8 +1589,16 @@ void SLtt_smart_puts(unsigned short *neww, unsigned short *oldd, int len, int ro
 #endif
    /* Find where the last non-blank character on old/new screen is */
 
+   space_char = ' ';
    if ((*(pmax-1) & 0xFF) == ' ')
      {
+	/* If we get here, then we can erase to the end of the line to create
+	 * the final space.  However, this will only work _if_ erasing will 
+	 * get us the correct color.  If the terminal supports BCE, then this
+	 * is easy.  If it does not, then we can only perform this operation
+	 * if the color is known via something like COLORFGBG.  For now, 
+	 * I just will not perform the optimization for such terminals.
+	 */
 	if ((Can_Background_Color_Erase)
 	    && SLtt_Use_Ansi_Colors)
 	  space_char = *(pmax - 1);
@@ -1641,8 +1643,8 @@ void SLtt_smart_puts(unsigned short *neww, unsigned short *oldd, int len, int ro
 	/* Try use use erase to bol if possible */
 	if ((Del_Bol_Str != NULL) && ((*neww & 0xFF) == 32))
 	  {
-	     unsigned short *p1;
-	     unsigned short blank;
+	     SLsmg_Char_Type *p1;
+	     SLsmg_Char_Type blank;
 
 	     p1 = neww;
 	     if ((Can_Background_Color_Erase)
