@@ -18,10 +18,6 @@
 #endif
 #define PI 3.14159265358979323846264338327950288
 
-#ifndef HAVE_STDLIB_H
-  extern double atof ();
-#endif
-
 #if defined(__unix__)
 #include <signal.h>
 #include <errno.h>
@@ -90,12 +86,41 @@ static int double_math_op_result (int op, unsigned char a, unsigned char *b)
    return 1;
 }
 
+#ifdef HAVE_ASINH
+# define ASINH_FUN	asinh
+#else
+# define ASINH_FUN	my_asinh
+static double my_asinh (double x)
+{
+   return log (x + sqrt (x*x + 1));
+}
+#endif
+#ifdef HAVE_ACOSH
+# define ACOSH_FUN	acosh
+#else
+# define ACOSH_FUN	my_acosh
+static double my_acosh (double x)
+{
+   return log (x + sqrt(x*x - 1));     /* x >= 1 */
+}
+#endif
+#ifdef HAVE_ATANH
+# define ATANH_FUN	atanh
+#else
+# define ATANH_FUN	my_atanh
+static double my_atanh (double x)
+{
+   return 0.5 * log ((1.0 + x)/(1.0 - x)); /* 0 <= x^2 < 1 */
+}
+#endif
+
 static int double_math_op (int op,
 			   unsigned char type, VOID_STAR ap, unsigned int na,
 			   VOID_STAR bp)
 {
    double *a, *b;
    unsigned int i;
+   double (*fun) (double);
 
    (void) type;
    a = (double *) ap;
@@ -107,101 +132,69 @@ static int double_math_op (int op,
 	return 0;
 
       case SLMATH_SINH:
-	for (i = 0; i < na; i++)
-	  b[i] = sinh (a[i]);
+	fun = sinh;
 	break;
-
-#ifdef HAVE_ASINH
-      case SLMATH_ASINH:
-	for (i = 0; i < na; i++)
-	  b[i] = asinh (a[i]);
-	break;
-#endif
-
       case SLMATH_COSH:
-	for (i = 0; i < na; i++)
-	  b[i] = cosh (a[i]);
+	fun = cosh;
 	break;
-
-#ifdef HAVE_ACOSH
-      case SLMATH_ACOSH:
-	for (i = 0; i < na; i++)
-	  b[i] = acosh (a[i]);
-	break;
-#endif
       case SLMATH_TANH:
-	for (i = 0; i < na; i++)
-	  b[i] = tanh (a[i]);
+	fun = tanh;
 	break;
-#ifdef HAVE_ATANH
-      case SLMATH_ATANH:
-	for (i = 0; i < na; i++)
-	  b[i] = atanh (a[i]);
-	break;
-#endif
       case SLMATH_TAN:
-	for (i = 0; i < na; i++)
-	  b[i] = tan (a[i]);
+	fun = tan;
 	break;
-
       case SLMATH_ASIN:
-	for (i = 0; i < na; i++)
-	  b[i] = asin (a[i]);
+	fun = asin;
 	break;
-
       case SLMATH_ACOS:
-	for (i = 0; i < na; i++)
-	  b[i] = acos (a[i]);
+	fun = acos;
 	break;
-
       case SLMATH_ATAN:
-	for (i = 0; i < na; i++)
-	  b[i] = atan (a[i]);
+	fun = atan;
 	break;
-
       case SLMATH_EXP:
-	for (i = 0; i < na; i++)
-	  b[i] = exp (a[i]);
+	fun = exp;
 	break;
-
       case SLMATH_LOG:
-	for (i = 0; i < na; i++)
-	  b[i] = log (a[i]);
+	fun = log;
 	break;
-
       case SLMATH_LOG10:
-	for (i = 0; i < na; i++)
-	  b[i] = log10 (a[i]);
+	fun = log10;
 	break;
-
       case SLMATH_SQRT:
-	for (i = 0; i < na; i++)
-	  b[i] = sqrt (a[i]);
+	fun = sqrt;
 	break;
-
       case SLMATH_SIN:
-	for (i = 0; i < na; i++)
-	  b[i] = sin (a[i]);
+	fun = sin;
+	break;
+      case SLMATH_COS:
+	fun = cos;
 	break;
 
-      case SLMATH_COS:
-	for (i = 0; i < na; i++)
-	  b[i] = cos (a[i]);
+      case SLMATH_ASINH:
+	fun = ASINH_FUN;
+	break;
+      case SLMATH_ATANH:
+	fun = ATANH_FUN;
+	break;
+      case SLMATH_ACOSH:
+	fun = ACOSH_FUN;
 	break;
 
       case SLMATH_CONJ:
       case SLMATH_REAL:
 	for (i = 0; i < na; i++)
 	  b[i] = a[i];
-	break;
-
+	return 1;
       case SLMATH_IMAG:
 	for (i = 0; i < na; i++)
 	  b[i] = 0.0;
-	break;
-
+	return 1;
      }
 
+   for (i = 0; i < na; i++)
+     b[i] = (*fun) (a[i]);
+
    return 1;
 }
 
@@ -211,112 +204,82 @@ static int float_math_op (int op,
 {
    float *a, *b;
    unsigned int i;
+   double (*fun) (double);
 
    (void) type;
    a = (float *) ap;
    b = (float *) bp;
 
+
    switch (op)
      {
       default:
 	return 0;
 
       case SLMATH_SINH:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)sinh (a[i]);
+	fun = sinh;
 	break;
-
-#ifdef HAVE_ASINH
-      case SLMATH_ASINH:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)asinh (a[i]);
-	break;
-#endif
-
       case SLMATH_COSH:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)cosh (a[i]);
+	fun = cosh;
 	break;
-
-#ifdef HAVE_ACOSH
-      case SLMATH_ACOSH:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)acosh (a[i]);
-	break;
-#endif
       case SLMATH_TANH:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)tanh (a[i]);
+	fun = tanh;
 	break;
-#ifdef HAVE_ATANH
-      case SLMATH_ATANH:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)atanh (a[i]);
-	break;
-#endif
       case SLMATH_TAN:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)tan (a[i]);
+	fun = tan;
 	break;
-
       case SLMATH_ASIN:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)asin (a[i]);
+	fun = asin;
 	break;
-
       case SLMATH_ACOS:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)acos (a[i]);
+	fun = acos;
 	break;
-
       case SLMATH_ATAN:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)atan (a[i]);
+	fun = atan;
 	break;
-
       case SLMATH_EXP:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)exp (a[i]);
+	fun = exp;
 	break;
-
       case SLMATH_LOG:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)log (a[i]);
+	fun = log;
 	break;
-
       case SLMATH_LOG10:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)log10 (a[i]);
+	fun = log10;
 	break;
-
       case SLMATH_SQRT:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)sqrt (a[i]);
+	fun = sqrt;
 	break;
-
       case SLMATH_SIN:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)sin (a[i]);
+	fun = sin;
+	break;
+      case SLMATH_COS:
+	fun = cos;
 	break;
 
-      case SLMATH_COS:
-	for (i = 0; i < na; i++)
-	  b[i] = (float)cos (a[i]);
+      case SLMATH_ASINH:
+	fun = ASINH_FUN;
+	break;
+      case SLMATH_ATANH:
+	fun = ATANH_FUN;
+	break;
+      case SLMATH_ACOSH:
+	fun = ACOSH_FUN;
 	break;
 
       case SLMATH_CONJ:
       case SLMATH_REAL:
 	for (i = 0; i < na; i++)
-	  b[i] = (float)a[i];
-	break;
-
+	  b[i] = a[i];
+	return 1;
       case SLMATH_IMAG:
 	for (i = 0; i < na; i++)
-	  b[i] = (float)0.0;
-	break;
-
+	  b[i] = 0.0;
+	return 1;
      }
 
+   for (i = 0; i < na; i++)
+     b[i] = (float) (*fun) ((double) a[i]);
+
    return 1;
 }
 
@@ -327,6 +290,7 @@ static int generic_math_op (int op,
    double *b;
    unsigned int i;
    SLang_To_Double_Fun_Type to_double;
+   double (*fun) (double);
    unsigned int da;
    char *a;
 
@@ -342,132 +306,55 @@ static int generic_math_op (int op,
 	return 0;
 
       case SLMATH_SINH:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = sinh (to_double((VOID_STAR) a));
-	     a += da;
-	  }
+	fun = sinh;
 	break;
-#ifdef HAVE_ASINH
-      case SLMATH_ASINH:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = asinh (to_double((VOID_STAR) a));
-	     a += da;
-	  }
-	break;
-#endif
       case SLMATH_COSH:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = cosh (to_double((VOID_STAR) a));
-	     a += da;
-	  }
+	fun = cosh;
 	break;
-#ifdef HAVE_ACOSH
-      case SLMATH_ACOSH:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = acosh (to_double((VOID_STAR) a));
-	     a += da;
-	  }
-	break;
-#endif
       case SLMATH_TANH:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = tanh (to_double((VOID_STAR) a));
-	     a += da;
-	  }
+	fun = tanh;
 	break;
-#ifdef HAVE_ATANH
-      case SLMATH_ATANH:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = atanh (to_double((VOID_STAR) a));
-	     a += da;
-	  }
-	break;
-#endif
       case SLMATH_TAN:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = tan (to_double((VOID_STAR) a));
-	     a += da;
-	  }
+	fun = tan;
 	break;
-
       case SLMATH_ASIN:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = asin (to_double((VOID_STAR) a));
-	     a += da;
-	  }
+	fun = asin;
 	break;
-
       case SLMATH_ACOS:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = acos (to_double((VOID_STAR) a));
-	     a += da;
-	  }
+	fun = acos;
 	break;
-
       case SLMATH_ATAN:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = atan (to_double((VOID_STAR) a));
-	     a += da;
-	  }
+	fun = atan;
 	break;
-
       case SLMATH_EXP:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = exp (to_double((VOID_STAR) a));
-	     a += da;
-	  }
+	fun = exp;
 	break;
-
       case SLMATH_LOG:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = log (to_double((VOID_STAR) a));
-	     a += da;
-	  }
+	fun = log;
 	break;
-
       case SLMATH_LOG10:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = log10 (to_double((VOID_STAR) a));
-	     a += da;
-	  }
+	fun = log10;
 	break;
-
       case SLMATH_SQRT:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = sqrt (to_double((VOID_STAR) a));
-	     a += da;
-	  }
+	fun = sqrt;
 	break;
-
       case SLMATH_SIN:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = sin (to_double((VOID_STAR) a));
-	     a += da;
-	  }
+	fun = sin;
+	break;
+      case SLMATH_COS:
+	fun = cos;
 	break;
 
-      case SLMATH_COS:
-	for (i = 0; i < na; i++)
-	  {
-	     b[i] = cos (to_double((VOID_STAR) a));
-	     a += da;
-	  }
+      case SLMATH_ASINH:
+	fun = ASINH_FUN;
 	break;
+      case SLMATH_ATANH:
+	fun = ATANH_FUN;
+	break;
+      case SLMATH_ACOSH:
+	fun = ACOSH_FUN;
+	break;
+
 
       case SLMATH_CONJ:
       case SLMATH_REAL:
@@ -476,14 +363,20 @@ static int generic_math_op (int op,
 	     b[i] = to_double((VOID_STAR) a);
 	     a += da;
 	  }
-	break;
+	return 1;
 
       case SLMATH_IMAG:
 	for (i = 0; i < na; i++)
 	  b[i] = 0.0;
-	break;
+	return 1;
      }
 
+   for (i = 0; i < na; i++)
+     {
+	b[i] = (*fun) (to_double ((VOID_STAR) a));
+	a += da;
+     }
+   
    return 1;
 }
 
@@ -512,6 +405,7 @@ static int complex_math_op (int op,
    double *a, *b;
    unsigned int i;
    unsigned int na2 = na * 2;
+   double *(*fun) (double *, double *);
 
    (void) type;
    a = (double *) ap;
@@ -520,85 +414,17 @@ static int complex_math_op (int op,
    switch (op)
      {
       default:
-      case SLMATH_ATANH:
-      case SLMATH_ACOSH:
-      case SLMATH_ASINH:
 	return 0;
 
       case SLMATH_REAL:
 	for (i = 0; i < na; i++)
 	  b[i] = a[2 * i];
-	break;
+	return 1;
 
       case SLMATH_IMAG:
 	for (i = 0; i < na; i++)
 	  b[i] = a[2 * i + 1];
-	break;
-
-      case SLMATH_EXP:
-	for (i = 0; i < na2; i += 2)
-	  SLcomplex_exp (b + i, a + i);
-	break;
-
-      case SLMATH_LOG:
-	for (i = 0; i < na2; i += 2)
-	  SLcomplex_log (b + i, a + i);
-	break;
-
-      case SLMATH_LOG10:
-	for (i = 0; i < na2; i += 2)
-	  SLcomplex_log10 (b + i, a + i);
-	break;
-
-      case SLMATH_SQRT:
-	for (i = 0; i < na2; i += 2)
-	  SLcomplex_sqrt (b + i, a + i);
-	break;
-
-      case SLMATH_SIN:
-	for (i = 0; i < na2; i += 2)
-	  SLcomplex_sin (b + i, a + i);
-	break;
-
-      case SLMATH_COS:
-	for (i = 0; i < na2; i += 2)
-	  SLcomplex_cos (b + i, a + i);
-	break;
-
-      case SLMATH_SINH:
-	for (i = 0; i < na2; i += 2)
-	  SLcomplex_sinh (b + i, a + i);
-	break;
-
-      case SLMATH_COSH:
-	for (i = 0; i < na2; i += 2)
-	  SLcomplex_cosh (b + i, a + i);
-	break;
-
-      case SLMATH_TANH:
-	for (i = 0; i < na2; i += 2)
-	  SLcomplex_tanh (b + i, a + i);
-	break;
-
-      case SLMATH_TAN:
-	for (i = 0; i < na2; i += 2)
-	  SLcomplex_tan (b + i, a + i);
-	break;
-
-      case SLMATH_ASIN:
-	for (i = 0; i < na2; i += 2)
-	  SLcomplex_asin (b + i, a + i);
-	break;
-
-      case SLMATH_ACOS:
-	for (i = 0; i < na2; i += 2)
-	  SLcomplex_acos (b + i, a + i);
-	break;
-
-      case SLMATH_ATAN:
-	for (i = 0; i < na2; i += 2)
-	  SLcomplex_atan (b + i, a + i);
-	break;
+	return 1;
 
       case SLMATH_CONJ:
 	for (i = 0; i < na2; i += 2)
@@ -606,9 +432,61 @@ static int complex_math_op (int op,
 	     b[i] = a[i];
 	     b[i+1] = -a[i+1];
 	  }
+	return 1;
+
+      case SLMATH_ATANH:
+	fun = SLcomplex_atanh;
+	break;
+      case SLMATH_ACOSH:
+	fun = SLcomplex_acosh;
+	break;
+      case SLMATH_ASINH:
+	fun = SLcomplex_asinh;
+	break;
+      case SLMATH_EXP:
+	fun = SLcomplex_exp;
+	break;
+      case SLMATH_LOG:
+	fun = SLcomplex_log;
+	break;
+      case SLMATH_LOG10:
+	fun = SLcomplex_log10;
+	break;
+      case SLMATH_SQRT:
+	fun = SLcomplex_sqrt;
+	break;
+      case SLMATH_SIN:
+	fun = SLcomplex_sin;
+	break;
+      case SLMATH_COS:
+	fun = SLcomplex_cos;
+	break;
+      case SLMATH_SINH:
+	fun = SLcomplex_sinh;
+	break;
+      case SLMATH_COSH:
+	fun = SLcomplex_cosh;
+	break;
+      case SLMATH_TANH:
+	fun = SLcomplex_tanh;
+	break;
+      case SLMATH_TAN:
+	fun = SLcomplex_tan;
+	break;
+      case SLMATH_ASIN:
+	fun = SLcomplex_asin;
+	break;
+      case SLMATH_ACOS:
+	fun = SLcomplex_acos;
+	break;
+      case SLMATH_ATAN:
+	fun = SLcomplex_atan;
 	break;
      }
 
+   for (i = 0; i < na2; i += 2)
+     (void) (*fun) (b + i, a + i);
+
    return 1;
 }
 #endif
