@@ -287,7 +287,7 @@ static void readlink_cmd (char *s)
    char buf[2048];
    int n;
 
-   n = readlink (s, buf, sizeof (buf));
+   n = readlink (s, buf, sizeof (buf)-1);
    if (n == -1)
      {
 	_SLerrno_errno = errno;
@@ -295,7 +295,6 @@ static void readlink_cmd (char *s)
      }
    else
      {
-	if (n == sizeof (buf)) n--;
 	buf[n] = 0;
 	s = buf;
      }
@@ -613,7 +612,7 @@ static int build_dirlist (char *file, char *opt, char ***listp, unsigned int *nu
 # else
    h = HDIR_CREATE;
    cFileNames = 1;
-   rc = DosFindFirst(pat, &h, FILE_READONLY | FILE_DIRECTORY |=20
+   rc = DosFindFirst(pat, &h, FILE_READONLY | FILE_DIRECTORY |
 		     FILE_ARCHIVED, &fd, sizeof(fd), &cFileNames, FIL_STANDARD);
    if (rc != 0)
      {
