@@ -177,6 +177,12 @@ int SLstruct_create_struct (unsigned int nfields,
 	VOID_STAR value;
 	char *name = field_names [i];
 
+	if (name == NULL)
+	  {
+	     SLang_verror (SL_INVALID_PARM, "A struct field name cannot be NULL");
+	     goto return_error;
+	  }
+
 	if (NULL == (f->name = SLang_create_slstring (name)))
 	  goto return_error;
 
@@ -577,6 +583,30 @@ int _SLstruct_define_typedef (void)
    return 0;
 }
 
+static int
+struct_datatype_deref (unsigned char stype)
+{
+   (void) stype;
+   
+   if (SLang_peek_at_stack () == SLANG_ARRAY_TYPE)
+     {
+	SLang_Array_Type *at;
+	int status;
+
+	if (-1 == SLang_pop_array_of_type (&at, SLANG_STRING_TYPE))
+	  return -1;
+	
+	status = SLstruct_create_struct (at->num_elements,
+					 (char **) at->data, NULL, NULL);
+
+	SLang_free_array (at);
+	return status;
+     }
+
+   SLang_push_integer (SLang_Num_Function_Args);
+   return _SLstruct_define_struct ();
+}
+
 static int register_struct (void)
 {
    SLang_Class_Type *cl;
@@ -587,6 +617,7 @@ static int register_struct (void)
    (void) SLclass_set_destroy_function (cl, struct_destroy);
    (void) SLclass_set_push_function (cl, struct_push);
    cl->cl_dereference = struct_dereference;
+   cl->cl_datatype_deref = struct_datatype_deref;
 
    cl->cl_foreach_open = struct_foreach_open;
    cl->cl_foreach_close = struct_foreach_close;
